import React, { useState } from 'react';
import { Dialog, DialogContent } from '../components';
import { SettingsPanel } from './SettingsPanel';
import { MonitorType, Sidebar } from './components/Sidebar';

type Props = {
  open: boolean;
  onClose: () => void;
};

const reports = ['Sequence 1', 'Sequence 2'];
const alerts = ['Potential New Incident', 'Potential New Info'];

export function InsightDialog(props: Props) {
  const [adding, setAdding] = useState<boolean>(false);
  const [active, setActive] = useState<string>('');
  const [activeCategory, setActiveCategory] = useState<MonitorType | undefined>(undefined);

  const handleChangeActive = (category: MonitorType, name: string) => {
    setActive(name);
    setActiveCategory(category);
  };

  return (
    <Dialog
      open={props.open}
      onOpenChange={(ret) => {
        if (!ret) props.onClose();
      }}
    >
      <DialogContent className="w-5/6 h-5/6 rounded-none py-0 px-0">
        <div className="flex w-full h-full">
          <Sidebar
            reports={reports}
            alerts={alerts}
            changeActive={handleChangeActive}
            setAdding={setAdding}
            setActiveCategory={setActiveCategory}
          />
          <SettingsPanel
            activeCategory={activeCategory}
            active={active}
            adding={adding}
            setAdding={setAdding}
            setActiveCategory={setActiveCategory}
          />
        </div>
      </DialogContent>
    </Dialog>
  );
}
