import {
  useQuerybuilderAttributesShown,
  useQuerybuilderGraph,
  useQuerybuilderHash,
  useQuerybuilderUnionTypes,
} from '@graphpolaris/shared/lib/data-access';
import {
  setQuerybuilderGraphology,
  toQuerybuilderGraphology,
  attributeShownToggle,
  setQueryUnionType,
  QueryUnionType,
} from '@graphpolaris/shared/lib/data-access/store/querybuilderSlice';
import React, { useMemo, useRef, useState } from 'react';
import { Button } from '@graphpolaris/shared/lib/components/buttons';
import { Icon } from '@graphpolaris/shared/lib/components/icon';
import { TextInput } from '@graphpolaris/shared/lib/components/inputs';
import { EntityPill } from '@graphpolaris/shared/lib/components/pills';
import { Handle, Position, useUpdateNodeInternals } from 'reactflow';
import { NodeAttribute, SchemaReactflowEntityNode, toHandleId } from '../../../model';
import { PillAttributes } from '../../pillattributes/PillAttributes';
import { DropdownTrigger, DropdownContainer, DropdownItemContainer, DropdownItem } from '@graphpolaris/shared/lib/components/dropdowns';
import { PopoverContext } from '@graphpolaris/shared/lib/components/layout/Popover';
import { useDispatch } from 'react-redux';
import { isEqual } from 'lodash-es';
import { getDataTypeIcon } from '@graphpolaris/shared/lib/components/DataTypeIcon';

/**
 * Component to render an entity flow element
 * @param {NodeProps} param0 The data of an entity flow element.
 */
export const QueryEntityPill = React.memo((node: SchemaReactflowEntityNode) => {
  const updateNodeInternals = useUpdateNodeInternals();
  const ref = useRef<HTMLDivElement | null>(null);

  const data = node.data;
  if (!data.leftRelationHandleId) throw new Error('EntityFlowElement: data.leftRelationHandleId is undefined');
  if (!data.rightRelationHandleId) throw new Error('EntityFlowElement: data.rightRelationHandleId is undefined');

  const graph = useQuerybuilderGraph();
  const attributeEdges = useMemo(
    () => graph.edges.filter((edge) => edge.source === node.id && !!edge?.attributes?.sourceHandleData.attributeType),
    [graph],
  );

  const unionType = useQuerybuilderUnionTypes()[node.id];

  return (
    <div className="w-fit h-fit nowheel" ref={ref} id="asd">
      <EntityPill
        title={
          <div className="flex flex-row justify-between items-center">
            <span>{data.name || ''}</span>
          </div>
        }
        withHandles="horizontal"
        handleLeft={
          <Handle
            id={toHandleId(data.leftRelationHandleId)}
            position={Position.Left}
            className={'!rounded-none !bg-transparent !w-full !h-full !border-0 !right-0 !left-0'}
            type="target"
          ></Handle>
        }
        handleRight={
          <Handle
            id={toHandleId(data.rightRelationHandleId)}
            // id="entitySourceRight"
            position={Position.Right}
            className={'!rounded-none !bg-transparent !w-full !h-full !border-0 !right-0 !left-0'}
            type="source"
          ></Handle>
        }
      >
        {data?.attributes && (
          <PillAttributes
            node={node}
            attributes={data.attributes}
            attributeEdges={attributeEdges.map((edge) => edge?.attributes)}
            unionType={unionType}
          />
        )}
      </EntityPill>
    </div>
  );
});

QueryEntityPill.displayName = 'QueryEntityPill';

export default QueryEntityPill;
