import { AttributeType, AttributeTypeStats } from '../../statistics.types';

const initializeStatistics = <T extends AttributeType>(type: T): AttributeTypeStats<T> => {
  switch (type) {
    case 'string':
      return {
        uniqueItems: 0,
        values: [],
        mode: '',
      } as unknown as AttributeTypeStats<T>;
    case 'boolean':
      return {
        true: 0,
        false: 0,
      } as unknown as AttributeTypeStats<T>;
    case 'number':
      return {
        min: Infinity,
        max: -Infinity,
        average: 0,
      } as unknown as AttributeTypeStats<T>;
    case 'date':
    case 'time':
    case 'datetime':
    case 'timestamp':
      return {
        min: Infinity,
        max: -Infinity,
        range: 0,
      } as unknown as AttributeTypeStats<T>;
    case 'array':
      return {
        length: 0,
      } as unknown as AttributeTypeStats<T>;
    case 'object':
      return {
        length: 0,
      } as unknown as AttributeTypeStats<T>;
    default:
      throw new Error(`Unknown attribute type: ${type}`);
  }
};

export { initializeStatistics };
