import React, { useRef, useImperativeHandle, forwardRef } from 'react';
import { VisualizationPropTypes, VISComponentType, VisualizationSettingsPropTypes } from '../../common';
import { SettingsContainer } from '@graphpolaris/shared/lib/vis/components/config';
import html2canvas from 'html2canvas';
import { Input } from '@graphpolaris/shared/lib/components/inputs';

export interface Vis0DProps {
  title: string;
}

const settings: Vis0DProps = {
  title: '',
};

export interface Vis0DVisHandle {
  exportImageInternal: () => void;
}

const formatNumber = (number: number) => {
  return number.toLocaleString('de-DE');
};
const Vis0D = forwardRef<Vis0DVisHandle, VisualizationPropTypes<Vis0DProps>>(({ data, settings }, refExternal) => {
  const internalRef = useRef<HTMLDivElement>(null);
  useImperativeHandle(refExternal, () => ({
    exportImageInternal() {
      const captureImage = () => {
        const element = internalRef.current;
        if (element) {
          html2canvas(element, {
            backgroundColor: '#FFFFFF',
          })
            .then((canvas) => {
              const finalImage = canvas.toDataURL('image/png');
              const link = document.createElement('a');
              link.href = finalImage;
              link.download = 'Vis0D.png';
              document.body.appendChild(link);
              link.click();
              document.body.removeChild(link);
            })
            .catch((error) => {
              console.error('Error capturing image:', error);
            });
        } else {
          console.error('Container element not found');
        }
      };

      const renderCanvas = () => {
        requestAnimationFrame(() => {
          captureImage();
        });
      };

      renderCanvas();
    },
  }));

  // !FIXME: When stats pills are ready, substitue results accordingly
  return (
    <div className="h-full w-full flex flex-col items-center justify-center overflow-hidden" ref={internalRef}>
      {settings.title && <span className="text-3xl text-center mb-4">{settings.title}</span>}
      {data?.nodes?.length > 0 ? (
        <span className="text-4xl text-center">Select 0D data</span>
      ) : (
        <span className="text-8xl text-center">{formatNumber(1231312)}</span>
      )}{' '}
    </div>
  );
});

const Vis0DSettings = ({ settings, updateSettings }: VisualizationSettingsPropTypes<Vis0DProps>) => {
  return (
    <SettingsContainer>
      <Input type="text" label="Title" value={settings.title} onChange={(value) => updateSettings({ title: value as string })} />
    </SettingsContainer>
  );
};

const Vis0DRef = React.createRef<Vis0DVisHandle>();

export const Vis0DComponent: VISComponentType<Vis0DProps> = {
  displayName: '0Dvis',
  description: 'KPI visualization',
  component: React.forwardRef((props: VisualizationPropTypes<Vis0DProps>, ref) => <Vis0D {...props} ref={Vis0DRef} />),
  settingsComponent: Vis0DSettings,
  settings: settings,
  exportImage: () => {
    if (Vis0DRef.current) {
      Vis0DRef.current.exportImageInternal();
    } else {
      console.error('0Dvis reference is not set.');
    }
  },
};

export default Vis0DComponent;
